'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var async = require('async');
var assert = require('assert');
var _ = require('lodash');
var Connection = require('./utils/connection');
var Strider = require('./stride').Strider;
var modeTypes = require('./utils/constants').mode;
var statuses = require('./utils/constants').statuses;

var _getDocs = Symbol('getDocs');
var _update = Symbol('update');
var _setSteps = Symbol('setSteps');
var _validator = Symbol('validator');
var _migratecb = Symbol('migratecb');
var _processor = Symbol('processor');
var _processSteps = Symbol('processSteps');
var _checkSumValidator = Symbol('checkSumValidator');
var _stepDecisionMaker = Symbol('stepDecisionMaker');
var _setConnection = Symbol('setConnection');

/**
 * Class to conduct the migration process.
 */

var Migration = function () {
    function Migration(config) {
        _classCallCheck(this, Migration);

        assert.notEqual(config.migrationCollection, null);
        this.db = null;
        this.config = config;
        this.up = [];
        this.down = [];
        this.steps = [];
        this.expired = [];
        this.migrationArray = [];
        this.collection = config.migrationCollection;
    }
    // public methods


    _createClass(Migration, [{
        key: 'add',
        value: function add(stepsList) {
            var buildNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

            this.migrationArray = this.migrationArray.concat(stepsList);
            this.buildNumber = buildNumber;
        }
    }, {
        key: 'migrate',
        value: async function migrate(processedCB) {
            try {
                await this[_setConnection]();
                this[_setSteps]();
                await this[_validator]();
                this[_processSteps](processedCB);
            } catch (err) {
                console.error('An error occurred while migration', err);
                this[_migratecb](err, processedCB);
            }
        }
    }, {
        key: 'removeExpiredVersions',
        value: async function removeExpiredVersions() {
            try {
                await this[_setConnection]();
                var expDocs = await this[_getDocs](true);
                expDocs = expDocs.map(function (doc) {
                    return doc.id;
                });
                if (expDocs.length) {
                    await this.db.collection(this.collection).deleteMany({ id: { $in: expDocs } });
                    this.driver.close();
                    return 'Expired docs removed from ' + this.collection + ' collection successfully';
                }
                this.driver.close();
                return 'No expired docs in ' + this.collection + ' collection';
            } catch (err) {
                console.log('Error occurred while removing expired docs from migration collection', err);
                throw err;
            }
        }
        // private methods

    }, {
        key: _setConnection,
        value: async function value() {
            try {
                this.driver = new Connection(this.config);
                this.db = await this.driver.connect();
            } catch (err) {
                console.log('Error occurred while connecting with Mongo server', err);
                throw err;
            }
        }
    }, {
        key: _getDocs,
        value: async function value() {
            var expired = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

            try {
                var docs = await this.db.collection(this.collection).find({ hasExpired: expired }).toArray();
                return docs;
            } catch (err) {
                console.log('Error occurred while fetching required docs', err);
                throw err;
            }
        }
    }, {
        key: _setSteps,
        value: function value() {
            var _this = this;

            this.migrationArray.forEach(function (obj) {
                var stride = new Strider(obj).readNcrypt().getStride();
                stride.buildNumber = _this.buildNumber;
                _this[stride.mode].push(stride);
            });
        }
    }, {
        key: _update,
        value: async function value(step, cb) {
            try {
                var id = step.id,
                    checksum = step.checksum,
                    buildNumber = step.buildNumber,
                    hasExpired = step.hasExpired,
                    rollbackStatus = step.rollbackStatus,
                    migrationStatus = step.migrationStatus;

                await this.db.collection(this.collection).updateOne({ id: step.id }, {
                    $set: {
                        id: id, checksum: checksum, buildNumber: buildNumber, hasExpired: hasExpired, rollbackStatus: rollbackStatus, migrationStatus: migrationStatus,
                        lastUpdated: new Date().toJSON().slice(0, 10).split('-').reverse().join('/')
                    }
                }, { upsert: true });
                cb();
            } catch (err) {
                step.status = statuses.error;
                cb('[" + step.id + "] failed to save the migration status: ' + err);
            }
        }
    }, {
        key: _processor,
        value: function value(step, cb) {
            var _this2 = this;

            if (step.mode === modeTypes.expired) {
                step.status = modeTypes.expired;
                step.hasExpired = true;
                this[_update](step, cb);
            } else if (step.mode === modeTypes.up || step.mode === modeTypes.down) {
                var modeType = step.mode === modeTypes.up ? statuses.migrationStatus : statuses.rollbackStatus;
                var otherModeType = step.mode === modeTypes.up ? statuses.rollbackStatus : statuses.migrationStatus;
                step[step.mode](this.db, function (err) {
                    if (err) {
                        step.status = statuses.error;
                        step[modeType] = statuses.failed;
                    } else {
                        step.status = statuses.success;
                        step[modeType] = statuses.success;
                        step[otherModeType] = null;
                    }
                    _this2[_update](step, cb);
                });
            }
        }
    }, {
        key: _processSteps,
        value: function value(processedCB) {
            var _this3 = this;

            this.steps = [].concat(_toConsumableArray(this.up), _toConsumableArray(this.down), _toConsumableArray(this.expired));
            async.series(this.steps.map(function (step) {
                return function (cb) {
                    _this3[_processor](step, cb);
                };
            }), function (err) {
                if (err) {
                    console.error('An error occurred while processing the step', err);
                    throw err;
                }
                _this3[_migratecb](null, processedCB);
            });
        }
    }, {
        key: _migratecb,
        value: function value(err, processedCB) {
            var result = this.steps.map(function (step) {
                return { id: step.id, status: step.status };
            });
            result = result.length === 0 ? 'Zero(0) Migration Files Processed...' : result;
            this.driver.close();
            processedCB(err, result);
        }
    }, {
        key: _checkSumValidator,
        value: function value(doc, mode) {
            var step = _.filter(this[mode], function (o) {
                return o.id == doc.id;
            })[0];
            if (step.checksum !== doc.checksum) {
                throw new Error('Checksum validation failed for id ' + doc.id);
            }
        }
    }, {
        key: _stepDecisionMaker,
        value: function value(arr, doc) {
            arr.forEach(function (el) {
                _.remove(el, function (obj) {
                    return obj.id === doc.id;
                });
            });
        }
    }, {
        key: _validator,
        value: async function value() {
            var _this4 = this;

            try {
                if (this.db) {
                    this.steps = [].concat(_toConsumableArray(this.up), _toConsumableArray(this.down), _toConsumableArray(this.expired));
                    var uniqueSteps = _.uniqBy(this.steps, 'id');
                    if (uniqueSteps.length < this.steps.length) {
                        throw new Error('Cannot proceed further as Migration files containing duplicate Ids');
                    }
                    var activeMigrationDocs = await this[_getDocs]();
                    var inactiveMigrationDocs = await this[_getDocs](true);
                    activeMigrationDocs.forEach(function (doc) {
                        var mode = doc.migrationStatus ? modeTypes.up : modeTypes.down;
                        if (_this4.config.validateChecksum) {
                            _this4[_checkSumValidator](doc, mode);
                        }
                        _this4[_stepDecisionMaker]([_this4[mode]], doc);
                    });
                    inactiveMigrationDocs.forEach(function (doc) {
                        _this4[_stepDecisionMaker]([_this4.expired, _this4.up, _this4.down], doc);
                    });
                }
            } catch (err) {
                console.error('An error occurred while steps validation', err);
                throw err;
            }
        }
    }]);

    return Migration;
}();

module.exports = {
    Migration: Migration
};