'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var assert = require('assert');
var MongoClient = require('mongodb').MongoClient;
/**
 * Class to create a connection with Mongo server 
 * as per the given configuration details
 */

var Connection = function () {
    function Connection(config) {
        _classCallCheck(this, Connection);

        if (config.mongoUri) {
            this.connectionUri = config.mongoUri;
        } else {
            assert.notEqual(config.host, null);
            this.host = config.host;
            this.db = config.db;
            this.user = config.user;
            this.password = config.password;
            this.replicaSet = config.replicaSet;
        }
    }

    _createClass(Connection, [{
        key: 'connect',
        value: function connect() {
            var _this = this;

            return new Promise(async function (res, rej) {
                try {
                    var uri = _this.connectionUri || _this.getConnectionUri();
                    _this.client = new MongoClient(uri, { useNewUrlParser: true });
                    var connectionInstance = await _this.client.connect();
                    var db = connectionInstance.db(_this.db);
                    res(db);
                } catch (err) {
                    console.error('An error occurred while connecting to Mongo Server: ' + err);
                    rej(err);
                }
            });
        }
    }, {
        key: 'close',
        value: function close() {
            this.client.close();
        }
    }, {
        key: 'getConnectionUri',
        value: function getConnectionUri() {
            var uri = 'mongodb://';
            if (this.user) {
                uri += this.user;
                if (this.password) {
                    uri += ':' + this.password;
                }
                uri += '@';
            }
            uri += this.host + '/';
            if (this.db) {
                uri += this.db;
            }
            if (this.replicaSet) {
                uri += '?replicaSet=' + this.replicaSet;
            }
            return uri;
        }
    }]);

    return Connection;
}();

module.exports = Connection;